@@ -4,7 +4,7 @@
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -31,6 +31,7 @@
 #include <libqmi-glib.h>
 
 #include "qmicli.h"
+#include "qmicli-helpers.h"
 
 /* Context */
 typedef struct {
@@ -48,6 +49,7 @@ static gboolean get_serving_system_flag;
 static gboolean get_system_info_flag;
 static gboolean get_technology_preference_flag;
 static gboolean get_system_selection_preference_flag;
+static gchar *set_system_selection_preference_str;
 static gboolean network_scan_flag;
 static gboolean reset_flag;
 static gboolean noop_flag;
@@ -81,6 +83,10 @@ static GOptionEntry entries[] = {
       "Get system selection preference",
       NULL
     },
+    { "nas-set-system-selection-preference", 0, 0, G_OPTION_ARG_STRING, &set_system_selection_preference_str,
+      "Set system selection preference",
+      "[cdma-1x|cdma-1xevdo|gsm|umts|lte|td-scdma]"
+    },
     { "nas-network-scan", 0, 0, G_OPTION_ARG_NONE, &network_scan_flag,
       "Scan networks",
       NULL
@@ -127,6 +133,7 @@ qmicli_nas_options_enabled (void)
                  get_system_info_flag +
                  get_technology_preference_flag +
                  get_system_selection_preference_flag +
+                 !!set_system_selection_preference_str +
                  network_scan_flag +
                  reset_flag +
                  noop_flag);
@@ -1098,7 +1105,7 @@ get_system_info_ready (QmiClientNas *client,
                     g_print ("\t\tPacket zone: '%" G_GUINT16_FORMAT "'\n", packet_zone);
                 if (network_id_valid) {
                     g_print ("\t\tMCC: '%s'\n", mcc);
-                    if (mnc[2] == 0xFF)
+                    if ((guchar)mnc[2] == 0xFF)
                         g_print ("\t\tMNC: '%.2s'\n", mnc);
                     else
                         g_print ("\t\tMNC: '%.3s'\n", mnc);
@@ -1261,14 +1268,14 @@ get_system_info_ready (QmiClientNas *client,
                 if (lac_valid)
                     g_print ("\t\tLocation Area Code: '%" G_GUINT16_FORMAT"'\n", lac);
                 if (cid_valid)
-                    g_print ("\t\tCell ID: '%" G_GUINT16_FORMAT"'\n", cid);
+                    g_print ("\t\tCell ID: '%u'\n", cid);
                 if (registration_reject_info_valid)
                     g_print ("\t\tRegistration reject: '%s' (%u)\n",
                              qmi_nas_network_service_domain_get_string (registration_reject_domain),
                              registration_reject_cause);
                 if (network_id_valid) {
                     g_print ("\t\tMCC: '%s'\n", mcc);
-                    if (mnc[2] == 0xFF)
+                    if ((guchar)mnc[2] == 0xFF)
                         g_print ("\t\tMNC: '%.2s'\n", mnc);
                     else
                         g_print ("\t\tMNC: '%.3s'\n", mnc);
@@ -1381,14 +1388,14 @@ get_system_info_ready (QmiClientNas *client,
                 if (lac_valid)
                     g_print ("\t\tLocation Area Code: '%" G_GUINT16_FORMAT"'\n", lac);
                 if (cid_valid)
-                    g_print ("\t\tCell ID: '%" G_GUINT16_FORMAT"'\n", cid);
+                    g_print ("\t\tCell ID: '%u'\n", cid);
                 if (registration_reject_info_valid)
                     g_print ("\t\tRegistration reject: '%s' (%u)\n",
                              qmi_nas_network_service_domain_get_string (registration_reject_domain),
                              registration_reject_cause);
                 if (network_id_valid) {
                     g_print ("\t\tMCC: '%s'\n", mcc);
-                    if (mnc[2] == 0xFF)
+                    if ((guchar)mnc[2] == 0xFF)
                         g_print ("\t\tMNC: '%.2s'\n", mnc);
                     else
                         g_print ("\t\tMNC: '%.3s'\n", mnc);
@@ -1495,14 +1502,14 @@ get_system_info_ready (QmiClientNas *client,
                 if (lac_valid)
                     g_print ("\t\tLocation Area Code: '%" G_GUINT16_FORMAT"'\n", lac);
                 if (cid_valid)
-                    g_print ("\t\tCell ID: '%" G_GUINT16_FORMAT"'\n", cid);
+                    g_print ("\t\tCell ID: '%u'\n", cid);
                 if (registration_reject_info_valid)
                     g_print ("\t\tRegistration reject: '%s' (%u)\n",
                              qmi_nas_network_service_domain_get_string (registration_reject_domain),
                              registration_reject_cause);
                 if (network_id_valid) {
                     g_print ("\t\tMCC: '%s'\n", mcc);
-                    if (mnc[2] == 0xFF)
+                    if ((guchar)mnc[2] == 0xFF)
                         g_print ("\t\tMNC: '%.2s'\n", mnc);
                     else
                         g_print ("\t\tMNC: '%.3s'\n", mnc);
@@ -1616,14 +1623,14 @@ get_system_info_ready (QmiClientNas *client,
                 if (lac_valid)
                     g_print ("\t\tLocation Area Code: '%" G_GUINT16_FORMAT"'\n", lac);
                 if (cid_valid)
-                    g_print ("\t\tCell ID: '%" G_GUINT16_FORMAT"'\n", cid);
+                    g_print ("\t\tCell ID: '%u'\n", cid);
                 if (registration_reject_info_valid)
                     g_print ("\t\tRegistration reject: '%s' (%u)\n",
                              qmi_nas_network_service_domain_get_string (registration_reject_domain),
                              registration_reject_cause);
                 if (network_id_valid) {
                     g_print ("\t\tMCC: '%s'\n", mcc);
-                    if (mnc[2] == 0xFF)
+                    if ((guchar)mnc[2] == 0xFF)
                         g_print ("\t\tMNC: '%.2s'\n", mnc);
                     else
                         g_print ("\t\tMNC: '%.3s'\n", mnc);
@@ -1707,8 +1714,7 @@ get_technology_preference_ready (QmiClientNas *client,
             &preference,
             NULL)) {
         preference_string = qmi_nas_radio_technology_preference_build_string_from_mask (preference);
-        g_print ("\tPersistent: '%s', duration: '%s'\n",
-                 qmi_device_get_path_display (ctx->device),
+        g_print ("\tPersistent: '%s'\n",
                  preference_string);
         g_free (preference_string);
     }
@@ -1868,6 +1874,89 @@ get_system_selection_preference_ready (QmiClientNas *client,
     shutdown (TRUE);
 }
 
+static QmiMessageNasSetSystemSelectionPreferenceInput *
+set_system_selection_preference_input_create (const gchar *str)
+{
+    QmiMessageNasSetSystemSelectionPreferenceInput *input = NULL;
+    QmiNasRatModePreference pref;
+    GError *error = NULL;
+
+    if (!qmicli_read_rat_mode_pref_from_string (str, &pref)) {
+        g_printerr ("error: failed to parse mode pref\n");
+        return NULL;
+    }
+
+    input = qmi_message_nas_set_system_selection_preference_input_new ();
+    if (!qmi_message_nas_set_system_selection_preference_input_set_mode_preference (
+            input,
+            pref,
+            &error)) {
+        g_printerr ("error: couldn't create input data bundle: '%s'\n",
+                    error->message);
+        g_error_free (error);
+        qmi_message_nas_set_system_selection_preference_input_unref (input);
+        return NULL;
+    }
+
+    if (!qmi_message_nas_set_system_selection_preference_input_set_change_duration (
+            input,
+            QMI_NAS_CHANGE_DURATION_PERMANENT,
+            &error)) {
+        g_printerr ("error: couldn't create input data bundle: '%s'\n",
+                    error->message);
+        g_error_free (error);
+        qmi_message_nas_set_system_selection_preference_input_unref (input);
+        return NULL;
+    }
+
+    if (pref & (QMI_NAS_RAT_MODE_PREFERENCE_GSM |
+                QMI_NAS_RAT_MODE_PREFERENCE_UMTS |
+                QMI_NAS_RAT_MODE_PREFERENCE_LTE)) {
+        if (!qmi_message_nas_set_system_selection_preference_input_set_gsm_wcdma_acquisition_order_preference (
+                input,
+                QMI_NAS_GSM_WCDMA_ACQUISITION_ORDER_PREFERENCE_AUTOMATIC,
+                &error)) {
+            g_printerr ("error: couldn't create input data bundle: '%s'\n",
+                        error->message);
+            g_error_free (error);
+            qmi_message_nas_set_system_selection_preference_input_unref (input);
+            return NULL;
+        }
+    }
+
+    return input;
+}
+
+static void
+set_system_selection_preference_ready (QmiClientNas *client,
+                                       GAsyncResult *res)
+{
+    QmiMessageNasSetSystemSelectionPreferenceOutput *output = NULL;
+    GError *error = NULL;
+
+    output = qmi_client_nas_set_system_selection_preference_finish (client, res, &error);
+    if (!output) {
+        g_printerr ("error: operation failed: %s\n", error->message);
+        g_error_free (error);
+        shutdown (FALSE);
+        return;
+    }
+
+    if (!qmi_message_nas_set_system_selection_preference_output_get_result (output, &error)) {
+        g_printerr ("error: couldn't set operating mode: %s\n", error->message);
+        g_error_free (error);
+        qmi_message_nas_set_system_selection_preference_output_unref (output);
+        shutdown (FALSE);
+        return;
+    }
+
+    g_print ("[%s] System selection preference set successfully; replug your device.\n",
+             qmi_device_get_path_display (ctx->device));
+
+    qmi_message_nas_set_system_selection_preference_output_unref (output);
+    shutdown (TRUE);
+}
+
 static void
 network_scan_ready (QmiClientNas *client,
                     GAsyncResult *res)
@@ -2099,6 +2188,27 @@ qmicli_nas_run (QmiDevice *device,
         return;
     }
 
+    /* Request to set system_selection preference? */
+    if (set_system_selection_preference_str) {
+        QmiMessageNasSetSystemSelectionPreferenceInput *input;
+        g_debug ("Asynchronously setting system selection preference...");
+
+        input = set_system_selection_preference_input_create (set_system_selection_preference_str);
+        if (!input) {
+            shutdown (FALSE);
+            return;
+        }
+
+        qmi_client_nas_set_system_selection_preference (ctx->client,
+                                                        input,
+                                                        10,
+                                                        ctx->cancellable,
+                                                        (GAsyncReadyCallback)set_system_selection_preference_ready,
+                                                        NULL);
+        qmi_message_nas_set_system_selection_preference_input_unref (input);
+        return;
+    }
+
     /* Request to scan networks? */
     if (network_scan_flag) {
         g_debug ("Asynchronously scanning networks...");
