@@ -4,7 +4,7 @@
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -57,8 +57,8 @@ static gboolean noop_flag;
 
 static GOptionEntry entries[] = {
     { "wds-start-network", 0, 0, G_OPTION_ARG_STRING, &start_network_str,
-      "Start network",
-      "[APN]"
+      "Start network (Authentication, Username and Password are optional)",
+      "[(APN),(PAP|CHAP|BOTH),(Username),(Password)]"
     },
     { "wds-follow-network", 0, 0, G_OPTION_ARG_NONE, &follow_network_flag,
       "Follow the network status until disconnected. Use with `--wds-start-network'",
@@ -336,7 +336,7 @@ start_network_ready (QmiClientWds *client,
 #define VALIDATE_UNKNOWN(str) (str ? str : "unknown")
 
     g_print ("[%s] Network started\n"
-             "\tPacket data handle: %u\n",
+             "\tPacket data handle: '%u'\n",
              qmi_device_get_path_display (ctx->device),
              (guint)ctx->packet_data_handle);
 
@@ -596,14 +596,46 @@ qmicli_wds_run (QmiDevice *device,
 
         /* Use the input string as APN */
         if (start_network_str[0]) {
+            gchar **split;
+
+            split = g_strsplit (start_network_str, ",", 0);
+
             input = qmi_message_wds_start_network_input_new ();
-            qmi_message_wds_start_network_input_set_apn (input, start_network_str, NULL);
+            qmi_message_wds_start_network_input_set_apn (input, split[0], NULL);
+
+            if (split[1]) {
+                QmiWdsAuthentication qmiwdsauth;
+
+                /* Use authentication method */
+                if (g_ascii_strcasecmp (split[1], "PAP") == 0) {
+                    qmiwdsauth = QMI_WDS_AUTHENTICATION_PAP;
+                } else if (g_ascii_strcasecmp (split[1], "CHAP") == 0) {
+                    qmiwdsauth = QMI_WDS_AUTHENTICATION_CHAP;
+                } else if (g_ascii_strcasecmp (split[1], "BOTH") == 0) {
+                    qmiwdsauth = (QMI_WDS_AUTHENTICATION_PAP | QMI_WDS_AUTHENTICATION_CHAP);
+                } else {
+                    qmiwdsauth = QMI_WDS_AUTHENTICATION_NONE;
+                }
+
+                qmi_message_wds_start_network_input_set_authentication_preference (input, qmiwdsauth, NULL);
+
+                /* Username */
+                if (split[2] && strlen (split[2])) {
+                    qmi_message_wds_start_network_input_set_username (input, split[2], NULL);
+
+                    /* Password */
+                    if (split[3] && strlen (split[3])) {
+                        qmi_message_wds_start_network_input_set_password (input, split[3], NULL);
+                    }
+                }
+            }
+            g_strfreev (split);
         }
 
         g_debug ("Asynchronously starting network...");
         qmi_client_wds_start_network (ctx->client,
                                       input,
-                                      10,
+                                      45,
                                       ctx->cancellable,
                                       (GAsyncReadyCallback)start_network_ready,
                                       NULL);
